{Record Label Component

 Designer:  Craig Ward, 100554.2072@compuserve.com
 Date:      18/10/95
 Version:   1.1

 Function:  Displays the current position in a data-set

 Calling:   To update the current record position the developer must
            call something similar to:
                 recordPosition1.Dataset := Table1;
                 recordPosition1.UpdateLabel;
            Of course, the dataset could be specified in the object-
            inspector. I've found that the best place to call the updateLabel
            procedure from is the OnDataChange event for a TDataSource component.

 Notes:     This code is derived from an un-named source who contributed it
            to a compuserve forum.
*******************************************************************************}
unit Rck;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBiProcs, DBiTypes, StdCtrls, DBTables;

type
 TRecordPosition = class(TLabel)
  private
    { Private declarations }
    FDataSet: TDataSet;
    FrecordLabel: String;
    procedure SetDataset(Value: TDataset);
    procedure SetrecordLabel(Value: string);
    function FindRecordNumber: longint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure UpdateLabel;
    constructor Create(aOwner: TComponent); override;
  published
    { Published declarations }
    property Dataset: TDataset read FDataset write SetDataset;
    property RecordLabel: string read FRecordLabel write SetRecordLabel; 
end;

procedure Register;

{****************************************************************************}
implementation

{procedure to register this as a custom component}
procedure Register;
begin
  RegisterComponents('Samples', [TRecordPosition]);
end;

constructor TRecordPosition.Create(aOwner: TComponent);
begin
 inherited Create(aOwner);
  RecordLabel := 'Record';
end;

{set the table name}
procedure TRecordPosition.SetDataset(Value: TDataset);
begin
 if value <> FDataset then
  begin
   FDataset := Value;
  end;
end;

{set the record type}
procedure TRecordPosition.SetrecordLabel(Value: string);
begin
 if value <> FrecordLabel then
  begin
   FrecordLabel := Value;
  end;
end;

{find record number}
function TRecordPosition.FindRecordNumber: longint;
var
 cP: CurProps;
 rP: RECProps;
begin
 {Return 0 if dataset is not Paradox or dBase}
 Result := 0;

 with FDataset do
  begin
   if state = dsInactive then
    exit;

   {we need to make this call to grab the cursor's iSeqNums}
   Check(DBiGetCursorProps(Handle,cP));

   {synchronize the BDE cursor with the dataset's cursor}
   UpdateCursorPos;

   {fill rP with the current record's properties}
   Check(DBiGetRecord(Handle,DBiNOLOCK,nil,@rP));

   {what kind of dataset are we looking at?}
   case cP.iSeqNums of
    0:
     result := rP.iPhyRecNum; {dBase}
    1:
     result := rP.iSeqNum;    {Paradox}
   end;
  end;
end;

{proc to update the label}
procedure TRecordPosition.UpdateLabel;
var
 i: longint;
 s: string;
begin
 i := FindRecordNumber;
 s := FrecordLabel;
 self.caption := s+' '+IntToStr(i)+' of '+IntToStr(FDataset.RecordCount);
end;

end.
